# -*- coding: utf-8 -*-
"""
'Activité 1 : Tracking utilisant OpenCV - KCF Tracker'
Mathieu Nierenberger
"""
import cv2
import numpy as np

#####################   TRACKING   ###########################

tracker = cv2.TrackerKCF_create()
video = cv2.VideoCapture('voiture_240fps.mp4')    # importer vidéo
#video = cv2.VideoCapture(1) # lire webcam 0 ou 1

ok,frame=video.read()   # lit la 1ère image de la vidéo
cv2.imwrite('image1.jpg', frame)

# Codec pour la sortie vidéo et création objet VideoWriter :
fourcc = cv2.VideoWriter_fourcc(*'XVID')
out = cv2.VideoWriter('tracked.avi', fourcc, 30, (frame.shape[1], frame.shape[0]))
fps=video.get(cv2.CAP_PROP_FPS)    # récupère le framerate de la vidéo pour avoir une échelle de tps

xtrack=[]
ytrack=[]
time=[]
currenttime=0

bbox = cv2.selectROI(frame) # sélection region of interest (ROI) à suivre
ok = tracker.init(frame,bbox)   # initialisation position ROI -> bbox
while True:
   ok,frame=video.read() # récupère l'image de la vidéo avec le flag ok si récupéré
   if not ok:   # si l'image n'est pas récupérée correctement, stop
       print('Fin de l analyse de la vidéo (ou échec de la récupération d une image')
       break
   ok,bbox=tracker.update(frame)    # si tracking ok (flag), bbox=position tracker
   time.append(currenttime)
   if ok:
        (x,y,w,h)=[int(v) for v in bbox]    # récupère l'emplacement et la taille du ROI
        xtrack.append(x)
        ytrack.append(y)
        cv2.rectangle(frame,(x,y),(x+w,y+h),(0,255,0),2,1)  # dessine un rectangle autour du ROI
   else:
        cv2.putText(frame,'Error',(100,100),cv2.FONT_HERSHEY_SIMPLEX,1,(0,0,255),2) # inscrit Error sur le images où le ROI n'est pas détecté
        xtrack.append(0)
        ytrack.append(0)
   cv2.imshow('Tracking',frame)
   out.write(frame)
   currenttime+=1/fps  # incrémente la variable time
   if cv2.waitKey(1)==ord('q'):  # q pour quitter fenêtre vidéo
        break

video.release()
out.release()
cv2.destroyAllWindows() # ferme toutes les fenêtres openCV
[time,xtrack,ytrack]=np.asarray([time,xtrack,ytrack]) # convertit les données en np.array
np.savez('data_track', time=time, xtrack=xtrack, ytrack=ytrack) # Sauvegarde des données pour l'activité 2