# -*- coding: utf-8 -*-
"""
'Activité 2 : Traitement des données'
Mathieu Nierenberger
"""
import matplotlib.pyplot as plt		# pour le tracé des graphes
import numpy as np

#  Import des données de l'activité 1 :
data_track = np.load('data_track.npz')
time = data_track['time']
xtrack = data_track['xtrack']
ytrack = data_track['ytrack']

#####################"   TRAJECTOIRES ET POSITIONS   ###########################

# Conversion pixels -> m

scale = 1   # LIGNE A MODIFIER
xtrack=xtrack     # LIGNE A MODIFIER
ytrack=ytrack     # LIGNE A MODIFIER

plt.figure(0) 		# crée une nouvelle figure
plt.scatter(xtrack,ytrack,s=2) 	# Tracé de la trajectoire
plt.grid() 		# affiche une grille de fond
plt.xlabel("Position x (m)") 	# ajoute une étiquette pour les abscisses
plt.ylabel("Position y (m)") 	# ajoute une étiquette pour les abscisses
plt.title("Trajectoire du centre du ROI tracké") 	# Titre du graphique
plt.savefig('0 - Trajectoire.png', dpi=300)
plt.show()

plt.figure(1) 		# crée une nouvelle figure
plt.scatter(time,xtrack, label="x(t)",s=2) 	# Tracé de x(t)
plt.scatter(time,ytrack, label="y(t)",s=2) 	# Tracé de x(t)
plt.grid() 		# affiche une grille de fond
plt.legend()
plt.xlabel("Temps (s)") 	# ajoute une étiquette pour les abscisses
plt.ylabel("Position (m)") 	# ajoute une étiquette pour les abscisses
plt.title("x(t) et y(t) du centre du ROI tracké") 	# Titre du graphique
plt.savefig('1 - Positions.png', dpi=300)
plt.show()

#####################"   FILTRAGE   ###########################

def Filtre_MG(U,n):
    i_debut = (n-1) // 2 # le // arrondit le résultat à l'entier le plus proche
    Res = U[0:i_debut]
    filt = [sum(U[i-i_debut:i+i_debut+1]) / n for i in range(i_debut, len(U) - i_debut)]
    Res = np.append(Res,filt)
    Res = np.append(Res,U[len(U)-i_debut:len(U)])
    return Res

# Filtrage :
xtrackf=xtrack   # LIGNE A MODIFIER
ytrackf=ytrack   # LIGNE A MODIFIER

# Zoom et tracé de x(t) filtrés ou non pour voir l'influence du filtrage :
plt.figure(2) 		# crée une nouvelle figure
d=12*len(time)//20     # indice début zoom
f=14*len(time)//20     # indice fin zoom
plt.plot(time[d:f],xtrackf[d:f], label="x(t) filtré",marker='.') 	# Tracé de x(t) filtré
plt.plot(time[d:f],xtrack[d:f], label="x(t)",marker='.') 	# Tracé de x(t)
plt.grid() 		# affiche une grille de fond
plt.legend()
plt.xlabel("Temps (s)") 	# ajoute une étiquette pour les abscisses
plt.ylabel("Position (m)") 	# ajoute une étiquette pour les abscisses
plt.title("Zoom sur x(t) et filtrage") 	# Titre du graphique
plt.savefig('2 - Positions - zoom et filtrage.png', dpi=300)
plt.show()

#####################   VITESSES   ###########################

def derivee(x,t):
    dx = []
    for i in range(0, len(x)-1):
        dx.append(x[i])   # LIGNE A MODIFIER
    dx.append(dx[len(dx)-1])    # on conserve la valeur pour la dernière dérivée non définie avec Euler implicite
    return dx

# Calcul des vitesses vx(t) et vy(t) :
vx=derivee(xtrackf,time)
vy=derivee(ytrackf,time)

plt.figure(3) 		# crée une nouvelle figure
plt.scatter(time,vx, label="vx(t)",s=2) 	# Tracé de vx(t)
plt.scatter(time,vy, label="vy(t)",s=2) 	# Tracé de vy(t)
plt.grid() 		# affiche une grille de fond
plt.legend()
plt.xlabel("Temps (s)") 	# ajoute une étiquette pour les abscisses
plt.ylabel("Vitesse (m/s)") 	# ajoute une étiquette pour les abscisses
plt.title("Vitesses du centre du ROI tracké") 	# Titre du graphique
plt.savefig('3 - Vitesses.png', dpi=300)
plt.show()

#####################   SAUVEGARDE DES VITESSES LORS DE LA DECELERATION   ###########################
# Sélection de la phase de décélération :
# Choisir sur la courbe de vy(t) précédente les t (s) correspondant au début à la fin de la phase de décélération
td=0.5   # LIGNE A MODIFIER
tf=1.1   # LIGNE A MODIFIER

# Recherche des valeurs exactes correspondantes dans le vecteur time
def closest(lst, x):    # trouver l'index d'une valeur proche dans une liste 
     lst = np.asarray(lst)
     idx = (np.abs(lst - x)).argmin()
     return idx+1

idxd=closest(time,td)
print('tps début décélération = ',time[idxd],' s')
idxf=closest(time,tf)
print('tps fin décélération = ',time[idxf],' s')

# Définition des variables associées à la décélération :
td=time[idxd:idxf]
vxd=vx[idxd:idxf]
vyd=vy[idxd:idxf]
np.savez('data_speed', td=td, vxd=vxd, vyd=vyd) # Sauvegarde des données pour les activités 3 et 4